import { __decorate, __metadata } from "tslib";
import { Component, Prop, Watch } from 'vue-property-decorator';
import { Icon } from '../Icon';
import { IconButton } from '../Button';
import { RatingSize } from './Rating.types';
import BaseComponent from '../BaseComponent';
import { classNamesFunction, css, format } from '@uifabric-vue/utilities';
const getClassNames = classNamesFunction();
let RatingBase = class RatingBase extends BaseComponent {
    constructor() {
        super(...arguments);
        this.internalValue = this.rating;
    }
    get classNames() {
        const { disabled, readonly, theme } = this;
        return getClassNames(this.styles, {
            disabled,
            readOnly: readonly,
            theme,
        });
    }
    get ratingLevels() {
        const { min, max } = this;
        if (min < 0 || max <= min)
            return [];
        let i = min + 1;
        return Array(max - min).fill(undefined).map(() => i++);
    }
    get ariaLabel() {
        if (!this.ariaLabelFormat)
            return '';
        return this.ariaLabelFormat.replace('{0}', `${this.internalValue}`).replace('{1}', `${this.max}`);
    }
    getRatingIconName(ratingLevel) {
        return this.internalValue >= ratingLevel ? this.iconName : this.unselectedIconName;
    }
    getRatingFillPercentage(ratingLevel) {
        if (ratingLevel - this.internalValue <= 0)
            return 100;
        if (ratingLevel - this.internalValue > 1)
            return 0;
        if (ratingLevel - this.internalValue > 0)
            return (1 - (ratingLevel - this.internalValue)) * 100;
        return 0;
    }
    setRating(rating) {
        this.internalValue = rating;
    }
    onPropValueChanged(newValue) {
        this.internalValue = newValue;
    }
    onValueChanged(value) {
        this.$emit('input', value);
    }
    getStarId(id, starNum) {
        return `${id}-star-${starNum - 1}`;
    }
    render(h) {
        const { internalValue: displayRating, ariaLabelFormat, classNames, readonly, ariaLabel, max, size, disabled, iconName, unselectedIconName } = this;
        const id = 'Rating';
        const labelId = 'RatingLabel';
        const rootSizeClass = size === RatingSize.Large ? classNames.rootIsLarge : classNames.rootIsSmall;
        const $stars = this.ratingLevels.map(starNum => {
            const fillPercentage = this.getRatingFillPercentage(starNum);
            return h('button', {
                key: starNum,
                attrs: {
                    id: this.getStarId(id, starNum),
                    ...(starNum === Math.ceil(displayRating) && { 'data-is-current': true }),
                    disabled: !!(disabled || readonly),
                    role: 'presentation',
                    type: 'button',
                },
                class: css(classNames.ratingButton, size === RatingSize.Large ? classNames.ratingStarIsLarge : classNames.ratingStarIsSmall),
                on: {
                    focus: ev => this.setRating(starNum),
                    click: ev => this.setRating(starNum),
                },
            }, [
                h('span', {
                    class: classNames.labelText,
                    attrs: {
                        id: `${labelId}-${starNum}`,
                    },
                }, format(ariaLabelFormat || '', starNum, max)),
                h('div', { class: classNames.ratingStar }, [
                    h(Icon, {
                        class: classNames.ratingStarBack,
                        props: {
                            iconName: fillPercentage > 0 ? iconName : unselectedIconName,
                        },
                    }),
                    !disabled && h(Icon, {
                        class: classNames.ratingStarFront,
                        props: {
                            iconName: fillPercentage > 0 ? iconName : unselectedIconName,
                        },
                        style: { width: fillPercentage + '%' },
                    }),
                ]),
            ]);
        });
        return h('div', {
            ref: 'root',
            attrs: {
                ...this.$attrs,
                'aria-label': readonly ? ariaLabel : '',
            },
            class: css('ms-Rating-star', classNames.root, rootSizeClass),
        }, [
            h('div', {
                class: css(classNames.ratingFocusZone, rootSizeClass),
            }, $stars),
        ]);
    }
};
__decorate([
    Prop({ type: Number, default: RatingSize.Small }),
    __metadata("design:type", Number)
], RatingBase.prototype, "size", void 0);
__decorate([
    Prop({ type: Number, default: 0 }),
    __metadata("design:type", Number)
], RatingBase.prototype, "min", void 0);
__decorate([
    Prop({ type: Number, default: 10 }),
    __metadata("design:type", Number)
], RatingBase.prototype, "max", void 0);
__decorate([
    Prop({ type: Number, default: 0 }),
    __metadata("design:type", Number)
], RatingBase.prototype, "rating", void 0);
__decorate([
    Prop({ type: String, default: '' }),
    __metadata("design:type", String)
], RatingBase.prototype, "ariaLabelFormat", void 0);
__decorate([
    Prop({ type: String, default: 'FavoriteStarFill' }),
    __metadata("design:type", String)
], RatingBase.prototype, "iconName", void 0);
__decorate([
    Prop({ type: String, default: 'FavoriteStar' }),
    __metadata("design:type", String)
], RatingBase.prototype, "unselectedIconName", void 0);
__decorate([
    Prop({ type: Boolean, default: false }),
    __metadata("design:type", Boolean)
], RatingBase.prototype, "disabled", void 0);
__decorate([
    Prop({ type: Boolean, default: false }),
    __metadata("design:type", Boolean)
], RatingBase.prototype, "readonly", void 0);
__decorate([
    Watch('rating'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number]),
    __metadata("design:returntype", void 0)
], RatingBase.prototype, "onPropValueChanged", null);
__decorate([
    Watch('internalValue'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], RatingBase.prototype, "onValueChanged", null);
RatingBase = __decorate([
    Component({
        components: { Icon, IconButton },
        inheritAttrs: false,
    })
], RatingBase);
export { RatingBase };
//# sourceMappingURL=Rating.base.js.map